# Copyright 2024-SomItCoop SCCL(<https://gitlab.com/somitcoop>)
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl.html).
{
    "name": "Knowledge Pages API",
    "version": "12.0.1.2.1",
    "depends": [
        "base",
        "knowledge",
        "base_rest",
        "document_page",
        "document_page_tag",
        "auth_api_key",
        "knowledge_translatable_pages",
        "api_common_base",
    ],
    "author": """
        Som It Cooperatiu SCCL,
        Som Connexió SCCL,
        Odoo Community Association (OCA)
    """,
    "category": "Knowledge",
    "website": "https://gitlab.com/somitcoop/erp-research/odoo-helpdesk",
    "license": "AGPL-3",
    "summary": """
        REST API for Knowledge Pages
    """,
    "data": [
        "views/knowledge_page_view.xml",
        "views/knowledge_page_highlight.xml",
        "security/ir.model.access.csv",
    ],
    "application": False,
    "installable": True,
}



from odoo import models, fields


class KnowledgePageHighlight(models.Model):
    _name = 'knowledge.page.highlight'
    _description = 'Knowledge Page Highlight'

    name = fields.Char(string='Name', required=True, translate=True)
    code = fields.Char(string='Code', required=True, help="Unique code to identify the highlight.")
    color = fields.Integer(string='Color Index')
    active = fields.Boolean(default=True)
    page_ids = fields.Many2many(
        'document.page',
        relation='page_highlight_rel',
        column1='highlight_id',
        column2='page_id',
        string='Pages'
    )

    _sql_constraints = [
        ('unique_name', 'unique(name)', 'Tags name must me unique'),
        ('unique_code', 'unique(code)', 'Tags code must me unique'),
    ]



# -*- coding: utf-8 -*-
from odoo.http import request
from odoo.exceptions import ValidationError
from odoo.addons.base_rest import restapi
from odoo.addons.component.core import Component
from werkzeug.urls import url_decode
import logging


_logger = logging.getLogger(__name__)


class KnowledgeCategoriesAPI(Component):
    _name = 'knowledge.categories.api'
    _inherit = 'base.rest.service'
    _usage = 'knowledge'
    _default_auth = "api_key"
    _collection = 'api_common_base.services'
    _description = """
    Knowledge Categories API
    Access to the knowledge categories with GET method
    """

    def _get_query_schema_general(self):
        return {
            "lang":
                {
                    "type"      :  "string",
                    "required"  :  False,
                    "allowed"   :  self.env['res.lang'].search(
                        [
                            ('translatable', '=', True),
                            ('active', '=', True)
                        ]
                    ).mapped(lambda x: x.code),
                },
            "category_initial":
                {
                    "type"      :  "string",
                    "required"  :  False,
                },
        }

    def _get_query_schema_specific(self):
        to_return = self._get_query_schema_general()
        del to_return['category_initial']
        return to_return

    #   --  CATEGORIES ENDPOINT  --

    @restapi.method([(["/categories/"], "GET")], input_param=restapi.CerberusValidator("_get_query_schema_general"))  # noqa
    def get_categories(self, category_initial=None, lang=None):
        return self._get_categories_internally(category_initial=category_initial, lang=lang)


    @restapi.method([(["/categories/<string:reference>"], "GET")], input_param=restapi.CerberusValidator("_get_query_schema_specific"))  # noqa
    def get_categories_by_reference(self, reference, lang=None):
        return self._get_categories_internally(category_initial=reference, lang=lang)

    def _get_categories_internally(self, category_initial=None, lang=None):

        domain = [
            ('type',                '=',    'category'),
            ('is_api_available',    '=',    True),
        ]
        if category_initial:
            # For a specific category
            domain.append(
                ('api_reference', '=', category_initial)
            )
        else:
            # For the general use...
            domain.append(
                ('parent_id', '=', False)
            )
        categories = self.env['document.page'].search(domain, limit=bool(category_initial))
        if not categories:
            raise ValidationError("Record doesn't exists or is unavaliable")
        result = []
        for category in categories:
            result.append(
                category.export_api_json(unfold=False, lang=lang)[0]
            )
        _logger.debug(result)
        return request.make_json_response(result)

    #   --  PAGES ENDPOINT  --

    @restapi.method(
        [(["/page/<string:reference>"], "GET")],
        input_param=restapi.CerberusValidator("_get_query_schema_specific")
    )
    def get_page(self, reference, lang=None):
        page = self.env['document.page'].search([('api_reference', '=', reference)], limit=1)
        if not page.exists() or not page.is_api_available or page.type != 'content':
            raise ValidationError("Record doesn't exist or is unavaliable")
        result = page.export_api_json(unfold=True, lang=lang)[0]
        _logger.debug(result)
        return request.make_json_response(result)

    @restapi.method(
        [(["/page/tag/<string:tag>"], "GET")],
        input_param=restapi.CerberusValidator("_get_query_schema_specific")
    )
    def get_page_by_tag(self, tag, lang=None):
        ctx = {'lang': lang or self.env.user.lang}

        tag_ids = self.env['document.page.tag'].with_context(ctx).search(
            [('name', '=ilike', tag)]
        ).ids
        if not tag_ids:  # ORM won't return archived tags, no need to check for active
            raise ValidationError("No active tag was found with the name '%s'" % tag)

        pages = self.env['document.page'].with_context(ctx).search([
            ('tag_ids', 'in', tag_ids),
            ('is_api_available', '=', True),
            ('type', '=', 'content')
        ])

        # We should check the parenting path so it is eligible
        pages = pages.filtered(lambda page: page._is_this_api_available()[0])
        if not pages:
            raise ValidationError("Record doesn't exists or is unavaliable")

        result = [
            page.export_api_json(
                unfold=False,
                lang=lang,
                breadcrumbs=True
            )[0] for page in pages
        ]
        _logger.debug(result)
        return request.make_json_response(result)

    #   --  SEARCH ENDPOINT  --

    @restapi.method([(["/search/<string:keyword>"], "GET")], input_param=restapi.CerberusValidator("_get_query_schema_specific"))  # noqa
    def search_page(self, keyword, lang=None):
        keyword = [a for a in url_decode(keyword).keys()][0]  # decoding the search term keyword
        if lang:
            pages_object = self.env['document.page'].with_context({'lang': lang})
        else:
            pages_object = self.env['document.page'].with_context({'lang': self.env.user.lang})

        pages = pages_object.search(
            [
                ('type', '=', 'content'),
                ('is_api_available', '=', True),
                ('name', 'ilike', keyword),
            ]
        )
        pages = pages | pages_object.search_translated_api_content(keyword)
        if not pages:
            raise ValidationError("Record doesn't exists or is unavaliable")
        # We should check the parenting path so it is eligible
        pages = pages.filtered(lambda page: page._is_this_api_available()[0])
        result = [page.export_api_json(unfold=False, lang=lang, breadcrumbs=True)[0] for page in pages]
        _logger.debug(result)
        return request.make_json_response(result)

    #   --  PAGES BY HIGHLIGHTED ENDPOINT  --

    @restapi.method([(["/highlighted/<string:highlighted_code>"], "GET")], input_param=restapi.CerberusValidator("_get_query_schema_specific"))  # noqa
    def get_highlighted_pages(self, highlighted_code, lang=None):
        highlight = self.env['knowledge.page.highlight'].search([('code', '=', highlighted_code)], limit=1)
        if not highlight.exists() or not highlight.active:
            raise ValidationError("The specified highlight does not exist or is inactive.")

        page_ids = highlight.page_ids
        if not page_ids:
            raise ValidationError("No pages found for the highlighted code.")
        page_ids = page_ids.filtered(lambda page: page._is_this_api_available()[0])
        if not page_ids:
            raise ValidationError("No pages found for the highlighted code.")
        result = [page.export_api_json(unfold=True, lang=lang, breadcrumbs=True)[0] for page in page_ids]

        _logger.debug(result)
        return request.make_json_response(result)



from odoo import models, fields, api, _
from odoo.exceptions import ValidationError

import re
import logging

_logger = logging.getLogger(__name__)

try:
    from jinja2.lexer import name_re as old_name_re
    import re

    name_re = re.compile(u'^%s$' % old_name_re.pattern)

except Exception:
    _logger.error("Jinja2 is not available")


class DocumentPage(models.Model):
    _inherit = "document.page"
    _order = 'api_sequence'

    _sql_constraints = [
        (
            "unique_api_reference",
            "UNIQUE(api_reference)",
            _("API Reference field must be unique."),
        )
    ]

    is_api_available = fields.Boolean(
        string="Is available in API",
    )
    api_reference = fields.Char(
        compute='_compute_slug_api_reference',
        string="API Reference",
        help="The reference used to point to a particular Page or Category",
        store=True,
        readonly=False
    )
    api_sequence = fields.Integer(
        string="API Sequence"
    )
    api_highlighted_ids = fields.Many2many(
        'knowledge.page.highlight',
        relation='page_highlight_rel',
        column1='page_id',
        column2='highlight_id',
        string='Highlighted'
    )

    @api.multi
    def _compute_slug_api_reference(self):
        for record in self:
            if not record.name:
                continue
            slug = self._string_to_slug(
                record.name +
                ((" " + record.parent_id.name) if record.parent_id else "")
            )
            domain = []
            i = 0
            if record.id:
                domain.append(('id', '!=', record.id))
            while self.search_count(domain + [('api_reference', '=', slug)]):
                i += 1
                slug = self._string_to_slug(
                    record.name +
                    ((" " + record.parent_id.name) if record.parent_id else "") +
                    " " + str(i)
                )
            record.api_reference = slug

    def _string_to_slug(self, s):
        """ Converts arbitrary string to slug. """
        # Remove symbols and replace spaces with underscores.
        cleaned = re.sub('[^\w\s-]', '', s).strip().lower()  # noqa
        slugified = re.sub('[-\s]+', '_', cleaned)  # noqa
        return slugified

    @api.constrains('api_reference')
    def _check_api_reference(self):
        for record in self:
            if record.is_api_available:
                if not record.api_reference:
                    continue
                if not name_re.match(record.api_reference):
                    raise ValidationError(_('API Reference is not valid'))
                if self.search([
                    ('api_reference', '=', record.api_reference),
                    ('id', '!=', record.id)
                ]):
                    raise ValidationError(_('API Reference must be unique'))

    @api.onchange('is_api_available')
    def _onchange_is_api_available(self):
        for record in self:
            if record.is_api_available and not record.api_reference:
                record._compute_slug_api_reference()

    @api.one
    def get_api_breadcrumb(self):
        """
        Retrieves a dictionary as a comprehensive breadcrumb with names and their references respectively.
        """
        breadcrumb = {
            "names" : [self.name],  # String sequence
            "references"   : [self.api_reference],    # Int sequence
        }
        if self.parent_id:
            parent_breadcrumb = self.parent_id.get_api_breadcrumb()[0]
            breadcrumb['names'] += parent_breadcrumb.get('names')
            breadcrumb['references'] += parent_breadcrumb.get('references')
        return breadcrumb

    @api.one
    def _is_this_api_available(self):
        # We need to check the tree for all the parents to be api_available
        if self.parent_id:
            return bool(
                self.is_api_available and self.parent_id._is_this_api_available()[0]
            )
        else:
            return self.is_api_available

    @api.one
    def export_api_json(self, unfold=False, lang=False, breadcrumbs=False):
        if not self._is_this_api_available()[0]:
            return []
        if lang:
            record = self.with_context({'lang': lang})
        else:
            record = self.with_context({'lang': self.env.user.lang})

        if record.type == "category":
            children = record.child_ids.filtered(lambda child: child.is_api_available)
            return {
                'reference'     : record.api_reference,
                'name'          : record.name,
                'type'          : record.type,
                'content'       : record.translated_api_content,
                'api_sequence'  : record.api_sequence,
                'childs'        : list(
                    child.export_api_json(unfold, record.env.context.get('lang'))[0] for child in children
                ),
            }
        elif record.type == "content":
            to_return = {
                'name'          : record.name,
                'reference'     : record.api_reference,
                'api_sequence'  : record.api_sequence,
                'type'          : record.type,
                'tags'          : [tag.name for tag in record.tag_ids],
            }

            # if not unfold then the flow comes from categories endpoint.
            if unfold:
                to_return.update({
                    'content'    : record.translated_api_content,
                    'parent_reference'  : record.parent_id.api_reference,
                })
            else:
                to_return.update({
                    'highlighted'    : [
                        [hl.name, hl.code] for hl in record.api_highlighted_ids
                    ]
                })

            # Now for the breadcrumbs
            if breadcrumbs:
                to_return.update({
                    'breadcrumb'    : record.get_api_breadcrumb()[0]
                })
            return to_return
